@@ -8,6 +8,7 @@ from typing import Callable, List, Optional, Tuple, Union
 from yellowdog_client import PlatformClient
 from yellowdog_client.common import SearchClient
 from yellowdog_client.model import (
+    AccountAllowance,
     AllowanceSearch,
     Application,
     ApplicationSearch,
@@ -27,6 +28,7 @@ from yellowdog_client.model import (
     MachineImageFamily,
     MachineImageFamilySearch,
     MachineImageFamilySummary,
+    MachineImageGroup,
     NamespaceSearch,
     ObjectPath,
     ObjectPathsRequest,
@@ -347,12 +349,13 @@ def get_worker_pools(
 
 
 @lru_cache
-def find_image_family_reference_by_name(
+def find_image_family_or_group_id_by_name(
     client: PlatformClient, image_family_name
 ) -> Optional[str]:
     """
     Resolve image family references. Complicated logic.
     Fully qualified name is used for non-ambiguous PRIVATE image families.
+    Return None if mapping can't be done, to be handled by the caller.
     """
 
     if image_family_name.startswith("ami-") or image_family_name.startswith(
@@ -369,8 +372,21 @@ def find_image_family_reference_by_name(
         if image_family_name.startswith("yd/")
         else image_family_name
     )
-    namespace, name = split_namespace_and_name(image_family_name)
 
+    # Handle image group names of form namespace/image-family-name/image-group-name
+    split_name = image_family_name.split("/")
+    if len(split_name) == 3:
+        try:
+            namespace, name = split_namespace_and_name(
+                f"{split_name[0]}/{split_name[1]}"
+            )
+            return client.images_client.get_image_group_by_name(
+                namespace=namespace, family_name=name, group_name=split_name[2]
+            ).id
+        except:
+            return
+
+    namespace, name = split_namespace_and_name(image_family_name)
     if_search = MachineImageFamilySearch(
         familyName=name,
         namespaces=None if namespace in [None, ""] else [namespace],
@@ -472,7 +488,7 @@ def clear_image_family_search_cache():
     """
     Clear the cache of Image Family name searches.
     """
-    find_image_family_reference_by_name.cache_clear()
+    find_image_family_or_group_id_by_name.cache_clear()
 
 
 def remove_allowances_matching_description(
@@ -589,7 +605,7 @@ def substitute_ids_for_names_in_crt(
 
     # Image family
     try:
-        crt.imagesId = _get_image_family_name_from_id(client, crt.imagesId)
+        crt.imagesId = _get_image_family_or_group_name_from_id(client, crt.imagesId)
     except:
         pass
 
@@ -599,7 +615,9 @@ def substitute_ids_for_names_in_crt(
             source.sourceTemplateId = _get_source_template_name_from_id(
                 client, source.sourceTemplateId
             )
-            source.imageId = _get_image_family_name_from_id(client, source.imageId)
+            source.imageId = _get_image_family_or_group_name_from_id(
+                client, source.imageId
+            )
     except:
         pass
 
@@ -618,14 +636,18 @@ def substitute_image_family_id_for_name_in_cst(
         return cst
 
     try:
-        cst.source.imageId = _get_image_family_name_from_id(client, cst.source.imageId)
+        cst.source.imageId = _get_image_family_or_group_name_from_id(
+            client, cst.source.imageId
+        )
         return cst
     except:
         pass
 
     try:
         # Google uses a different property name
-        cst.source.image = _get_image_family_name_from_id(client, cst.source.image)
+        cst.source.image = _get_image_family_or_group_name_from_id(
+            client, cst.source.image
+        )
         return cst
     except:
         pass
@@ -635,11 +657,17 @@ def substitute_image_family_id_for_name_in_cst(
 
 def substitute_id_for_name_in_allowance(
     client: PlatformClient,
-    allowance: Union[RequirementsAllowance, SourcesAllowance, SourceAllowance],
-) -> Union[RequirementsAllowance, SourcesAllowance, SourceAllowance]:
+    allowance: Union[
+        AccountAllowance, RequirementsAllowance, SourcesAllowance, SourceAllowance
+    ],
+) -> Union[AccountAllowance, RequirementsAllowance, SourcesAllowance, SourceAllowance]:
     """
     Substitute IDs in Allowance objects.
     """
+
+    if not ARGS_PARSER.substitute_ids:
+        return allowance
+
     if isinstance(allowance, RequirementsAllowance):
         allowance.requirementCreatedFromId = _get_requirement_template_name_from_id(
             client, allowance.requirementCreatedFromId
@@ -650,9 +678,7 @@ def substitute_id_for_name_in_allowance(
             client, allowance.sourceCreatedFromId
         )
 
-    elif isinstance(allowance, SourceAllowance):
-        # No processing for source allowances
-        pass
+    # No processing for other allowance types
 
     return allowance
 
@@ -696,22 +722,40 @@ def _get_requirement_template_name_from_id(
 
 
 @lru_cache
-def _get_image_family_name_from_id(
-    client: PlatformClient, image_family_id: Optional[str]
+def _get_image_family_or_group_name_from_id(
+    client: PlatformClient, image_family_or_group_id: Optional[str]
 ) -> Optional[str]:
     """
-    Obtain the namespace/name of an image family.
+    Obtain the namespace/name of an image family or image group.
     Otherwise, return the original value.
     """
-    if get_ydid_type(image_family_id) != YDIDType.IMAGE_FAMILY:
-        return image_family_id
-    try:
-        image_family: MachineImageFamily = client.images_client.get_image_family_by_id(
-            image_family_id
-        )
-        return f"yd/{image_family.namespace}/{image_family.name}"
-    except:
-        return image_family_id
+    if get_ydid_type(image_family_or_group_id) == YDIDType.IMAGE_FAMILY:
+        try:
+            image_family: MachineImageFamily = (
+                client.images_client.get_image_family_by_id(image_family_or_group_id)
+            )
+            return f"yd/{image_family.namespace}/{image_family.name}"
+        except:
+            return image_family_or_group_id
+
+    elif get_ydid_type(image_family_or_group_id) == YDIDType.IMAGE_GROUP:
+        try:
+            image_group: MachineImageGroup = client.images_client.get_image_group_by_id(
+                image_family_or_group_id
+            )
+            image_family: MachineImageFamily = (
+                client.images_client.get_image_family_by_id(
+                    # The image family ID can be derived from the group ID
+                    image_family_or_group_id.replace("imggrp", "imgfam").rsplit(":", 1)[
+                        0
+                    ]
+                )
+            )
+            return f"yd/{image_family.namespace}/{image_family.name}/{image_group.name}"
+        except:
+            return image_family_or_group_id
+
+    return image_family_or_group_id
 
 
 @lru_cache
